function [mom,momnorm] = moments(N,s,b,mu,sig)
%% MOMENTS returns N+1 1D (up to order N)
%   [mom,momnorm] = moments(N,s,b(1),b(2),mu,sig) return 1D N-order moments EX^N & 
%   normalized moments E[(X-mu)^N]/sig^N of distribution corresponding to s 
%   with mean mu, stdev sig over range (b(2)(1),b(2)(2))
%   
%   E.g., mean=mom(2), variance=mom(3)
%
%   Options:
%    s=1 (truncated) normal distribution with parameters mu, sig over range (b(1),b(2))
%    s=2 uniform distribution over range (b(1),b(2))

mom = [1,zeros(1,N)];
momnorm = [1,0,1,zeros(1,N-2)];
if N<2, return; end

disp(['Compute MOMENTS: order=',num2str(N),', s=',num2str(s(1)),', range (',num2str(b(1)),',',num2str(b(2)),')'])


switch s
    case 1
        if isequal(b,[-inf,inf]) % standard normal distribution N(0,1)
            mom=[1,0,1,0,zeros(1,N-3)]; % standard normal
            for k=4:2:N; % only even moments are nonzero
                fak=1;
                for i=2:2:k, fak=fak*(i-1); end % odd factorial
                mom(k+1)=fak;
            end
            momnorm=mom;
            if exist('mu','var') || exist('sig','var') 
                momraw=[1,zeros(1,N)]; % raw moments of N(mu,sig) computed from standard normal moments
                for k=1:N
                    for i=0:2:k
                        coeff = factorial(k)/(factorial(i)*factorial(k-i)); % nchoosek(k,i);
                        momraw(k+1)=momraw(k+1)+coeff*sig^i*mu^(k-i)*mom(i+1);  
                    end
                end
                mom=momraw;
            end
        else % truncated N(mu,sig) with standardized range alpha=(b(1)-mu)/sigma 
            if ~exist('mu','var'), mu=0; sig=1; end
            fak=[1,zeros(1,N)];
            if b(1)==-inf
                S=normcdf(b(2));
                fak(2)=-normpdf(b(2))/S;
            elseif b(2)==inf
                S=1-normcdf(b(1));
                fak(2)=normpdf(b(1))/S;
            else
                S=normcdf(b(2))-normcdf(b(1));
                fak(2)=-(normpdf(b(2))-normpdf(b(1)))/S;
            end
            for k=2:N
                if b(1)==-inf
                    fak(k+1)=-b(2)^(k-1)*normpdf(b(2))/S + (k-1)*fak(k+1-2);  % factor I_k, k>=2
                elseif b(2)==inf
                    fak(k+1)=b(1)^(k-1)*normpdf(b(1))/S + (k-1)*fak(k+1-2);  % factor I_k, k>=2
                else
                    fak(k+1)=-(b(2)^(k-1)*normpdf(b(2))-b(1)^(k-1)*normpdf(b(1)))/S + (k-1)*fak(k+1-2);  % factor I_k, k>=2
                end
            end
            for k=1:N
                for i=0:k
                    coeff = factorial(k)/(factorial(i)*factorial(k-i));
                    mom(k+1)=mom(k+1)+coeff*sig^i*mu^(k-i)*fak(i+1);  
                end
            end
            
            % normalization
            momcen=[1,zeros(1,N)]; 
            for k=1:N
                for i=0:k
                    coeff = factorial(k)/(factorial(i)*factorial(k-i));
                    %momraw(k+1)=momraw(k+1)+coeff*sig^i*mu^(k-i)*mom(i+1);  % ND centered moments
                    momcen(k+1)=momcen(k+1)+(-1)^(k-i)*coeff*mom(2)^(k-i)*mom(i+1);  % ND centered moments
                end
            end
            M=ones(1,N+1)./sqrt(momcen(3)).^(0:N);
            momnorm=momcen*diag(M);
        end
    case 2
        if exist('mu','var') && exist('sig','var')
            b(1)=mu-sqrt(3)*sig;
            b(2)=mu+sqrt(3)*sig;
        else
            mu=(b(2)-b(1))/2;
            sig=sqrt((b(2)-b(1))^2/12);
        end   
        % raw moments of UD with range (b(1),b(2))
        for k=1:N; % odd moments are 0
            for i=0:k, mom(k+1)=mom(k+1)+b(1)^i*b(2)^(k-i); end % odd factorial
            mom(k+1)=mom(k+1)/(k+1);
        end
        M=ones(1,N+1)./sqrt(mom(3)).^(0:N);
        momnorm=mom*diag(M); % standardized moments (i.e. moments of (X-mu)/sigma for mu==0))

    otherwise
        disp('Not implemented (moments.m)')
end

