function T = tensor_emp(tname,N,I,PHI,mom)
%% TENSOR_EMP compute inner products of tensor product of MD polynomial basis functions
%   A = TENSOR_EMP(name,N,I,PHI,mom) return rows of N-fold tensor indices 
%   with basis functions in (PHI,I).
%   Computed by moments substitution and stored in name.
%
%   REQUIRE: deg(P_i)*N=P*N 1D moments per dimension

global TENSORPATH

% check if tensor is already stored
if exist([TENSORPATH,tname], 'file')
        load([TENSORPATH,tname]);
        disp(['Load tensor ',[TENSORPATH,tname]]), return
end

xi=symvar(PHI);
ND=length(xi);
TK=length(I(:,1));
P=max(I(:,1));

if length(mom(1,:))<P*N, 
    disp(['ERROR: Require ',num2str(P*N),' moments per dimension (given ',num2str(length(mom(1,:))),').']); 
    return;
end

M = index_tensor(N,TK); % unique set of tensor indices sorted decreasingly

disp(['Compute TENSOR by moment subst.: N=',num2str(N),', total # basis functions TK=',num2str(TK),', ',num2str(length(M)),' rows remaining.']); tic

% preallocate tensor with correct dimensions (not necessary)
T = zeros(length(M),N+1);
T(1,:)=ones(1,N+1);

parfor j=2:length(M)  % parfor possible
    J = num2cell(M(j,:));
    t=1;
    for d=1:ND
        G=1;
        for n=1:N
            G=G*PHI(I( J{n},d)+1,d);
        end
        
        Int=expand(G);
        for m=length(mom)-1:-1:1 % norm of polynomials
            Int=subs(Int,xi(d)^m,mom(d,m+1));
        end
        
        if abs(vpa(Int,100))<10e-20
            %disp(['SMALL integral :',char(vpa(Int,10))]);
            Int=0;
        end
        t=t*Int;
    end
    T(j,:) = [M(j,:),t];
end

T=T(T(:,end)~=0,:); % only want to store nonzero tensor entries

disp(['TIME to create tensor is ',num2str(toc),' s, ',num2str(length(T)),' rows.'])
if ~isempty(tname)
    disp(['Saved ',[TENSORPATH,tname]]); save([TENSORPATH,tname],'T','PHI','I','mom');
end
end



function A = index_tensor (N,K)
%% INDEX_TENSOR Recursive index generation for PCE-tensor 
%   A = INDEXTENSOR(N,K) return sorted tensor indices for K elements of order N.
%
%   Fast & low memory storag property: Reduce total number of K^N permutations 
%   (==permutations of elements 1,..,K w/o repetition of elements)
%   using ortogonality.

A=(1:K)';
for j=N-1:-1:1
    Atemp=[];
    nrow=length(A(:,1));
    for k=1:K
        Atemp((k-1)*nrow+1:k*nrow,:)=[k*ones(nrow,1),A];
    end
    % sort the row sums
    row_totals = sum(Atemp,2);
    [~, row_ids] = sort(row_totals, 'ascend'); 
    Atemp=Atemp(row_ids,:);
    
    % sort single rows in descending order
    Atemp=sort(Atemp,2,'descend');
    % remove redundant index rows (due to symmetry)
    A=unique(Atemp,'rows');
end
end
