function [P,w,b] = basis(K,s,ext,b)
%% BASIS Polynomial chaos basis constrauction.
%   [P,w,lb,ub] = BASIS(K,s,ext,lb,ub) return basis polynomials, weights & bounds.
%
%   Warning: might require a second orthogonalization for more precision
%
%   Options:
%       Default values for ext is '' (no normalization).
%       s=1 (truncated) normal distribution with parameters mu, sig over range 
%       (lb,ub) (default: (-inf,inf))
%       s=2 uniform distribution over range (lb,ub) (default: (-1,1))
%       s=3 Chebyshev distribution over range (lb,ub) (default: (-1,1))

if ~exist('ext','var'), ext=''; end

ND=length(s);
xi = sym('xi',[1 ND]);
assumeAlso(xi,'real');
P=sym('P',[K+1 ND]);
P(1,:) = ones(1,ND);
P(2,:) = xi;
assumeAlso(P,'real');

for d=1:ND
    switch s(d)
        case 1 % normal distribution    
            if isempty(b), b=ones(ND,1)*[-inf,inf]; end
            if isequal(b(d,:),[-inf,inf]) 
                w(d) = 1/sqrt(2*pi)*exp(-xi(d)^2/2); %#ok<*AGROW>
                for i=3:K+1
                    P(i,d) = expand( xi(d)*P(i-1,d) - (i-2) * P(i-2,d));
                end
            else
                % truncated ND 
                S=normcdf(b(d,2))-normcdf(b(d,1));
                w(d) = 1/sqrt(2*pi)*exp(-xi(d)^2/2);
                w(d) = w(d)/S;
                A = (xi(d)*ones(K+1,1)).^([0:K]'); 
                P(:,d) = gs_ortho(A,w(d),b(d,:));
            end
        case 2 % uniform distribution over (-1,1)
            b(d,:) = [-1,1];
            w(d) = sym(1/2);
            for i=3:K+1
                P(i,d) = expand( (2*(i-2)+1)/(i-1) * xi(d)*P(i-1,d) - (i-2)/(i-1) * P(i-2,d));
            end
        case 3 % chebyshev over (-1,1)
            b(d,:) = [-1,1];
            w(d) = 1/sqrt(1-xi(d)*xi(d))*2/pi; %scaled (for PHI(0), w(d)=w(d)/2)
            for i=3:K+1
                P(i,d) = expand( 2*xi(d)*P(i-1,d) - P(i-2,d));
            end
        otherwise
            disp('Undefined basis type (basis.m)')
    end
    
    P(:,d) = expand(P(:,d));
    if strcmp(ext,'norm')
        P(:,d) = gs_norm(P(:,d),w(d),b(d,:));
    end
end

assumeAlso(P,'real')
end
