function T = tensor(tname,N,I,PHI,w,b)
%% TENSOR of inner products of basis functions Phi
%   A = TENSOR(name,N,I,PHI,w,lb,ub) return rows of indices corresponding
%   to nonzero entries of inner product rank N tensor of MD polynomial
%   basis functions Phi evaluated using (PHI,I).
%
% ` Memory-saving vector of rows format corresponding to all nonzero entries
%   of the N-fold tensor of dimension TK (TK=length(I)). TENSOR is stored in name.

global TENSORPATH

TEPS=10e-20;

TK=length(I(:,1));
xi=symvar(PHI);
ND=length(xi);

% check if tensor is already stored
if exist([TENSORPATH,tname], 'file'),load([TENSORPATH,tname]); disp(['Load tensor ',[TENSORPATH,tname]]), return; end

M = index_tensor(N,TK); % unique set of tensor indices (sorted sum increasing, columns decreasing)
disp(['Compute TENSOR by integration: N=',num2str(N),', TK=',num2str(TK),', ',num2str(length(M)),' rows remaining.']); tic
T = zeros(length(M),N+1);

for j=1:length(M)  % parfor possible
    J = M(j,:); % or J = num2cell(M(j,:))
    t=1;
    for d=1:ND
        G=1;
        for n=1:N
            % PHI(In,1)*...*PHI(In,d) contain 1D polys
            % In=I(J{n},d)+1 is the MD-index of the MD poly (which row in PHI)
            G=G*PHI(I(J(n),d)+1,d);
        end
        % compute tensors through 1D integration
        Int=int(expand(G)*w(d),xi(d),b(d,1),b(d,2));
        if abs(vpa(Int))<TEPS, Int=0; end
        t=t*Int;
    end
    T(j,:) = [M(j,:),t];
end
% remove zero tensor entries
T=T(T(:,end)~=0,:); 

disp(['TIME to create tensor is ',num2str(toc),' s, ',num2str(length(T)),' rows.'])
if ~isempty(tname), disp(['Saved ',[TENSORPATH,tname],'.']); save([TENSORPATH,tname],'T','PHI','w','b','I'); end
end


function A = index_tensor (N,K)
%% INDEX_TENSOR Recursive index generation for PCE-tensor 
%   A = INDEXTENSOR(N,K) return sorted (increasing) tensor indices for K elements of order N.
%
%   Fast & low memory storag property: Reduce total number of K^N permutations 
%   (==permutations of elements 1,..,K w/o repetition of elements)
%   using ortogonality.

A=(1:K)';
for j=N-1:-1:1
    Atemp=[];
    nrow=length(A(:,1));
    for k=1:K
        Atemp((k-1)*nrow+1:k*nrow,:)=[k*ones(nrow,1),A];
    end
    % sort the row sums
    row_totals = sum(Atemp,2);
    [~, row_ids] = sort(row_totals, 'ascend'); 
    Atemp=Atemp(row_ids,:);
    
    % sort single rows in descending order
    Atemp=sort(Atemp,2,'descend');
    % remove redundant index rows (due to symmetry)
    A=unique(Atemp,'rows');
end

end
