function [] = sample_pdf(name,I,s,ext,b,IND,MC)
%% Sample PDF given solution of PCE coefficients
%   [] = sample_pdf(name,I,s,ext,b,IND,MC,B)
%
% Input:    name        - solution struct from solved PCE-surrogate
%           I,s,ext,b   - index column vector (dim. [TK,ND])  (PCE-surr.)
%           s,ext,b     - basis type, norm. & bounds (PCE-surr.)
%           IND         - variable indices to plot
%           MC          - # of MC samples
%
%
%   Example run: sample_states('ccrane_nom',[0:9]',1,'',[],[1,6],1000)


if ~exist('MC','var'), MC = 100; end %sample more points
if ~exist('IND','var'), IND=1; end

%% extract required pce coefficient solution
[SD,~,~] = read_solution([name,'.txt']);
[Ngrid,NX]=size(SD);
NXD=floor(NX/length(I(:,1)));
II=IND;
for j=2:length(I(:,1))
    II=[II,IND+(j-1)*NXD];
end
SD = SD(:,II);

T=[3:ceil(Ngrid/2):Ngrid,Ngrid];
SD=SD(T,:);
T=T/Ngrid;

%% build pce surrogate
[S] = pce_build(SD,I,s,ext,b);

%% generate random samples
disp('MC Sampling')
Q = zeros(MC,length(s));
for d=1:length(s)
    if s(d) == 2
        C=sqrt(3); % 100%-CI for UD
        Q(:,d) = random('unif',-1, 1, MC,1);
    else
        C=2.576; % 99%-CI for SND
        Q(:,d) = random('norm',0, 1, MC,1);
    end
end

%% substitute sample in pce approximation
Xpdf = zeros(length(S(:,1)), length(IND), MC);
for m=1:MC
    Xpdf(:,:,m) = subs(S,symvar(S),squeeze(Q(m,:)));
end

%% plot histograms & estimators
[mom,~] = pce_moment(2,I,s,ext,b);
cmap=colormap(parula);
c1=cmap(1,:); c2=cmap(floor(length(cmap)/2),:); c3=cmap(end-10,:);
S=zeros(MC,length(T));
n=0;
for i = 1:length(IND)
    for t=1:length(T)
        S(:,t)=squeeze(Xpdf(t,i,:));
        n=n+1; subplot(length(IND),length(T),n,'LineWidth',1);
        hold on
        
        % histogram
        [f,x]=hist(S(:,t),50);
        fsc=trapz(x,f);
        bar(x,f/fsc,'facecol','no','edgecolor',c1)  % histogram normalized to area (sum over all bins is 1)
        y = ylim;
        % kde: [f,xgrid] = ksdensity(S(:,t)); plot(xgrid,f,'color',c2)
        
        % density estimators
        MOM=subs(mom,sort(symvar(mom)),SD(t,i:length(IND):end));
        meanSD=double(MOM(2));
        varSD=double(MOM(3)-MOM(2)*MOM(2));
        xgrid=meanSD-6*sqrt(varSD):sqrt(varSD)/100:meanSD+6*sqrt(varSD);
        switch s(1)
            case 1, plot(xgrid,normpdf(xgrid,meanSD,sqrt(varSD)),'LineWidth',2,'color',c2)
            case 2, plot(xgrid,unifpdf(xgrid,meanSD-C*sqrt(varSD),meanSD+C*sqrt(varSD)),'LineWidth',2,'color',c2)
        end
        ylim(y)
        title(sprintf(['xd=',num2str(IND(i)-1),' t=',num2str(T(t))]),'FontSize',12);
        
        line([1 1]*meanSD,y,'LineWidth',1,'Color',c3)
        line([1 1]*meanSD-C*sqrt(varSD),y,'LineWidth',.5,'Color',c3)
        line([1 1]*meanSD+C*sqrt(varSD),y,'LineWidth',.5,'Color',c3)
        hold off;
    end
end
end

